function img_out = specification(img_src,img_ref,N)
% specification modifies the histogram of a source image so that it become
% identical to the histogram of the reference image.
%
% img_out = specification(img_src,img_ref [,N])
%
% img_src   : source image, which histogram will be modified
% img_ref   : reference image (=> reference histogram).
% N         : (optional) is the number of bins for the histogram
%             computation. Default:256
%
% N.B.
% . PLEASE GIVE IMG_SRC AND IMG_REF IN [0,1]
% . img_src and img_ref can have different sizes. 
% . img_src and img_ref MUST BE 1-dimentional.
%
% REFERENCES:
%   Not the same, but related to the article "Midway Image Equalization"
%   from Thierry Guillemot and Julie Delon. (IPOL Preprint, August 7, 2015)
%
%   Note that the authors provide a solution for the quatization effect
%   that could appear: a dithering method (adding a gaussian noise with a
%   small standard deviation to the images). This is not implemented here.
%
% Function written by Charles Hessel on October 15, 2015.


if nargin == 2, N = 256; end

% --- WARNING: to ensure that the images are in [0,1] ---
%              because if they aren't, error at line 54

img_src = max(0,min(1,img_src));
img_ref = max(0,min(1,img_ref));

% --- Cumulative histograms computation ---

[counts,X] = imhist(img_ref,N);
Href = cumsum(counts)/numel(img_ref);

[counts,~] = imhist(img_src,N);
Hsrc = cumsum(counts)/numel(img_src);

% --- LooUp Table computation ---

lut = zeros(1,N);
for k=1:N
    [~,ind]= max(Href >= Hsrc(k));
    lut(k) = X(ind);
end

% --- LookUp Table application ---

img_out = lut(round((N-1)*img_src+1));

end